const req = {
    Check: function (val, item) { if (val === undefined || val === null) return true; return false; }
};
const checkMethod = {
    //Type: { Check: function (val, item) { return false; } },
    Min: {
        Check: function (val, item) { return !isFit.call(this, val, item.Min, '>'); },
        CreateText: function (val, item) {
            return { message: item.ErrText || ((item.Title || item.Name) + '必须大于' + item.Min), fun: 'Min', name: item.Name, item: item, value: val }
        }
    },
    Max: {
        Check: function (val, item) { return !isFit.call(this, val, item.Max, '<'); },
        CreateText: function (val, item) {
            return { message: item.ErrText || ((item.Title || item.Name) + '必须小于' + item.Max), fun: 'Max', name: item.Name, item: item, value: val }
        }
    },
    Length: {
        Check: function (val, item) { if (val.length !== undefined && val.length > item.Length) return true; },
        CreateText: function (val, item) {
            return { message: item.ErrText || ((item.Title || item.Name) + '等长度不能超过' + item.Length), fun: 'Length', name: item.Name, item: item, value: val }
        }
    },
    Regx: {
        Check: function (val, item) { return !new RegExp(item.Regx).test(val); },
        CreateText: function (val, item) {
            return { message: item.ErrText || ((item.Title || item.Name) + '格式不正确'), name: item.Name, fun: 'Regx', item: item, value: val }
        }
    },
    Check: {
        Check: function (val, item) { return isFit(val, item) === undefined },
        CreateText: function (val, item) {
            return { message: item.ErrText || ((item.Title || item.Name) + '不正确'), name: item.Name, fun: 'Check', item: item, value: val }
        }
    },
    CheckOptions: {
        Check: function (val, item) {
            if (item.Options && Array.isArray(item.Options)) {
                if (item.Options.indexOf && item.Options.indexOf(val)) return true;
                for (let i = 0, len = item.Options.length; i < len; i++) {
                    if (typeof val === 'object' && val.hasOwnProperty("Id")) {
                        if (item.Options[i].Id === val.Id) return true;
                    } else if (item.Options[i].Value === val) return true;
                }
            }
            return false;
        },
        CreateText: function (val, item) { { return `${item.Title || item.Name}不在可选范围内` } }
    }
}
const include = require('./include');
const Types = require('./types')
const put = require('./put');
const check = function (data, item) {
    if (item) {
        if (Array.isArray(item)) {
            let result = [];
            item.map(t => { let r = check.call(this, data, t); if (r !== undefined) result.push(r); });
            return result.length > 0 ? result : undefined;
        } else if (typeof item === 'object' && item.Name) {

            let val = put.get.call(this, item.Name, data);
            let isBlank = req.Check.call(this, val, item, data);

            if (item.Req && isBlank && (!item.FieldState || include(item.FieldState, data))) {
                return { message: item.ErrText || ("(1)必须填写" + (item.Title || item.Name)), name: item.Name, item: item, value: val, data: data }
            }
            if (!isBlank) {
                if (item.Pros && item.Pros.length > 0) {
                    let count = 0;
                    let error = [];
                    if (item.Pros.indexOf('array') > -1) {
                        count++;
                        if (!Array.isArray(val)) {
                            error.push('数组');
                        }
                    }
                    if (item.Pros.indexOf('object') > -1) {
                        count++;
                        if (typeof val !== 'object' || Array.isArray(val)) {
                            error.push('对象');
                        }
                    }
                    if (item.Pros.indexOf('function') > -1) {
                        count++;
                        if (typeof val !== 'function') {
                            error.push('方法');
                        }
                    }
                    if (count && error.length === count) return { message: (item.Title || item.Name) + "必须是" + error.join("或"), name: item.Name, item: item, value: val, data: data }
                }
                if (item.Type) {
                    if (Array.isArray(item.Type) && item.Type.length > 0) {
                        let isType;
                        for (let i = 0, len = item.Type.length; i < len; i++) {
                            if (Types[item.Type[i]] && Types[item.Type[i]].Is(val)) {
                                isType = Types[item.Type[i]]; break;
                            }
                        }
                        if (!isType && item.Req) return (item.Error ? item.Error['Type'] || item.Error.Default : undefined) || { message: ((item.Title || item.Name) + '不正确(1)'), name: item.Name, value: val, item: item }
                        if (item.Req && isType.IsEmpty(val)) return { message: item.ErrText || ((item.Title || item.Name) + '不能为空值'), name: item.Name, item: item }
                    } else if (Types[item.Type]) {
                        isBlank = Types[item.Type].IsEmpty(val);
                        if (item.Req && isBlank && (!item.FieldState || include(item.FieldState, data))) {
                            //isBlank = Types[item.Type].IsEmpty(val);
                            return { message: item.ErrText || ("(2)必须填写" + (item.Title || item.Name)), name: item.Name, item: item }
                        }
                        if (!isBlank && !Types[item.Type].Is(val)) {
                            isBlank = req.Check.call(this, val, item, data);
                            return (item.Error ? item.Error['Type'] || item.Error.Default : undefined) || { message: ((item.Title || item.Name) + '不正确(2)'), name: item.Name, value: val, item: item }
                        }

                    }
                }
                if (item.Childs && Array.isArray(item.Childs)) {
                    let e = check.call(this, val, item.Childs);
                    if (e) return { message: e.message, name: item.Name, value: val, item: item, err: e };
                } else {
                    for (let p in checkMethod) {
                        if (checkMethod.hasOwnProperty(p) && item.hasOwnProperty(p)) {
                            let err = checkMethod[p].Check.call(this, val, item, data);
                            if (err) return checkMethod[p].CreateText.call(this, val, item, data);
                        }
                    }
                }
            }
        }
    }
};
module.exports = check;