let get = require('./get');
let setClass = require('./set');
let set = setClass.set;
let Types = require('./types')
const getDefaultValue = function (DefaultValue, Item) {
    if (DefaultValue === undefined) return DefaultValue;
    switch (typeof DefaultValue) {
        case 'function':
            return DefaultValue.call(Item);
        case 'string':
            {
                if (DefaultValue === 'DEFAULT') {
                    if (Item.Type === 'Date')
                        return Types.Date.DefaultValue();
                    if (Item.Type === 'DateTime')
                        return Types.Datetime.DefaultValue();
                    return ''
                } else if (DefaultValue === 'EMPTY') {
                    if (Item.Type === 'Date')
                        return Types.Date.Empty();
                    if (Item.Type === 'DateTime')
                        return Types.Datetime.Empty();
                    return ''
                } else
                    return DefaultValue;
            }
        default:
            return DefaultValue;
    }
}

const getFormatValue = function (Value, Item) {
    if (Value === undefined) return Value;
    if (Item.Format) {
        switch (typeof Item.Format) {
            case 'function':
                Value = Item.Format.call(Item, Value);
                break;
            case 'string':
                if (Array.isArray(Value)) {
                    let v = Item.Format;
                    for (let i in Value) {
                        v = v.replace('$data' + i + "$", Value[i])
                    }
                    Value = v;
                } else if (typeof Value === 'object') {
                    let v = Item.Format;
                    for (let i in Value) {
                        if (Value.hasOwnProperty(i)) v = v.replace('$' + i + "$", Value[i])
                    }
                    Value = v;
                } else
                    Value = Item.Format.replace('$data$', Value);
                break;
        }
    }
    if (typeof Value === 'object' && Item.Create === true) Value = merge({}, Value);
    return Value;

}
const getFontBackValue = function (Value, Item) {
    if (Item && (Item.Front || Item.Back) && Value !== undefined && typeof Value === 'string') {
        if (Item.Front) Value = Item.Front + Value;
        if (Item.Back) Value += Item.Back;
    }
    return Value;
}
const getValueByType = function (Value, Item) {

    if (Array.isArray(Value) && (!Item || Item.Type !== 'Array')) {
        for (let i in Value) {
            Value[i] = getValueByType(Value[i], Item);
        }
        return Value;
    } else {
        if (Value === undefined) Value = getDefaultValue(Item.DefaultValue, Item);
        if (!Item.Type || !Types[Item.Type]) {
            return getFormatValue(Value, Item);
        }

        if (Value === undefined) return Value;
        let t = Types[Item.Type];
        if (t.Childs) {
            Value = put(Value, t.Childs);
        }

        Value = getFormatValue(Value, Item);
        if (Value === '$DEFAULTVALUE$') Value = t.DefaultValue();
        else if (Value === '$EMPTY$') Value = t.Empty();
        else {
            if (Value !== undefined) {
                Value = t.ConvertFromString(Value, Item.NullToEmpty, Item.ElementType);
            } else {
                Value = getDefaultValue(Item.DefaultValue, Item);
            }
            if (t.Format && Value !== undefined) Value = t.Format(Value)
            if (Value === undefined) return Value;
        }
        return getFontBackValue(Value, Item);
    }

    return Value;
}
const put = function (item, data, value) {
    if (!item) return value;
    if (Array.isArray(data)) {
        for (let i in data) {
            let v = put.call(this, item, data[i]);
            if (v !== undefined) {
                if (!Array.isArray(value)) value = [];
                value.push(v);
            }
        }
        return value;
    } else {
        if (Array.isArray(item)) {
            let vs = [];
            let num = 0;
            for (let i in item) {
                if (!value) value = {};
                let v = put.call(this, item[i], data, (item[i].Name ? value || {} : undefined));
                if (v !== undefined && !item[i].Name) vs.push(v);
            }
            if (vs.length > 0 && num > 0) {
                value._List_ = vs;
                return value
            };
            if (vs.length > 0) return vs;
            return value;
        } else if (typeof item === 'object' && (item.Path || item.Name)) {
            if (item.Path && Array.isArray(item.Path)) {
                let v;
                if (item.Split) {
                    //有split则按split组合成字串
                    let vs = [];
                    for (let i in item.Path) {
                        let v1 = put.call(this, item.Path[i], data);
                        if (v1 !== undefined) vs.push(v1);
                    }
                    if (vs.length > 0) v = vs.join(item.Split);
                } else if (item.ForOne) {
                    //有ForOne 则只取第一个有值的值
                    for (let i in item.Path) {
                        v = put.call(this, item.Path[i], data);
                        if (v !== undefined) break;
                    }
                } else {
                    //默认取出一个数组
                    v = [];
                    for (let i in item.Path) {
                        let v1 = put.call(this, item.Path[i], data);
                        if (v1 !== undefined) v.push(v1);
                    }
                }

                v = getValueByType(v, item);
                if (v !== undefined) {
                    if (item.Name) {
                        if (typeof value !== 'object') value = {};
                        value = set.call(this, item.Name, v, value);
                    } else return v;
                }
            } else if (item.Childs) {
                let v2 = getValueByType(put.call(this, item.Childs, data[item.Name]), item);
                if (v2 !== undefined) {
                    if (item.Name) {
                        value = set.call(this, item.Name, v2, value);
                    } else return v2;
                }
            } else {
                let v1 = put.call(this, (item.Path || item.Name), data);
                v1 = getValueByType(v1, item);

                if (v1 !== undefined) {
                    if (item.Name) {
                        if (typeof value !== 'object') value = value || {};
                        value = set.call(this, item.Name, v1, value);
                    } else value = v1;
                }
            }

        } else if (typeof item === 'string') {
            if(this && this.config && this.config.isLowerCase ===true) item = item.toLowerCase();
            return get.call(this, item, data);
        }
    }
    return value;
}
const putClass = class {
    constructor(storage, config) {
        //console.log('create get class..........................................................................')
        this.storage = storage;
        this.config = Object.assign({ StringSplit: ".", StorageLetter: "$" }, config);
    }
    put() { return put.apply(this, arguments); }
}
module.exports = { putClass: putClass, get: get };