let numberClass = require('./types/number');
let stringClass = require('./types/string');
let dateClass = require('./types/date');
let boolClass = require('./types/bool');
let guidClass = require('./types/guid');
let objectClass = require('./types/object');
let arrayClass = require('./types/array');

const Id = new numberClass('Id', 0, true);
const Int = new numberClass('Int', 0, false);
const Long = new numberClass('Long', 0, false);
const Link = new numberClass('Link', 0, true);
const Enum = new numberClass('Enum', 0, true);
const Decimal = new numberClass('Decimal', 2, true);
const Decimal4 = new numberClass('Decimal4', 4, true);
const Float = new numberClass('Float', 2, true);
const Float4 = new numberClass('Float4', 4, true);
const Char = new stringClass('Char', { maxlen: 20 });
const String = new stringClass('String', { maxlen: 50 });
const Html = new stringClass('Html', {});
const Image = new stringClass('Image', { maxlen: 300 });
const Images = new stringClass('Images', { maxlen: 3999 });
const File = new stringClass('File', { maxlen: 300 });
const Mobile = new stringClass('Mobile', { maxlen: 11, minlen: 11, regx: /^1[0-9]{10}$/ });
const IDCard = new stringClass('IDCard', { maxlen: 23, minlen: 18, regx: [/^[1-9]\d{5}(18|19|([23]\d))\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]/, /^[1-9]\\d{5}[1-9]\\d{3}((0\\d)|(1[0-2]))(([0|1|2]\\d)|3[0-1])\\d{3}[0-9Xx]$/] });
const Email = new stringClass('Email', { maxlen: 300, regx: /^\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$/ })
const Password = new stringClass('Password', { maxlen: 20, regx: /^[A-Za-z0-9$#@^&?.]{6,20}$/ })
const City = new stringClass('City', { maxlen: 100 });

const Date = new dateClass('Date', true);
const Datetime = new dateClass('Datetime', false);
const DateTime = new dateClass('DateTime', false);

const Bool = new boolClass('Bool');
const Boolean = new boolClass('Boolean');

const Guid = new guidClass('Guid');
const GUID = new guidClass('Guid');
const Object = new objectClass('Object', [{ Name: "Id", Req: true }, { Name: "Type" }]);
const Array = new arrayClass('Array');

const types = {
    Id, Bool, Boolean, Int,
    Long, Link, Enum, Decimal,
    Decimal4, Float, Float4, Char, String, Html, Image, Images,
    File, Mobile, IDCard, Email, Password, City, Date, Datetime,
    DateTime, Guid, GUID, Object,
    Array
}


module.exports = types;