const typeClass = require('./typeClass');
const Count = function (a, b, l) {
    if (typeof a !== 'number' || typeof b !== 'number') return undefined;
    let value;
    switch (l) {
        case '+':
            value = a + b;
            break;
        case '-':
            value = a - b;
            break;
        case '*':
            value = a * b;
            break;
        case '/':
            value = a / b;
            break;
    }
    //let k = dot > 0 ? Math.pow(10, dot) : 1;
    // if (k > 1) value = Math.round(value * k) / k;
    return value;
}
const Storage = {
    "$empty$": function () { return 0; },
    "$defaultvalue$": function () { return 0; }
}
class number extends typeClass {
    constructor(name, dot, noZero) {
        super(name);
        this.dot = dot;
        this.name = name;
        this.noZero = noZero || true;
    }
    get Dot() {
        return this.dot;
    }
    get BaseInt() {
        return this.dot ? Math.pow(10, this.dot) : 1;
    }
    get ConvertHandler() {
        return function (value) {
            return this.BaseInt > 1 ? Math.round(parseFloat(value) * this.BaseInt) / this.BaseInt : parseInt(value);
        }
    }
    get Empty() {
        return function () {
            return 0;
        }
    }
    get IsEmpty() {
        return function (value) {
            return value === NaN || value === 0;
        }
    }
    get Is() {
        return function (value) {
            if (typeof value === 'string') if (this.IsStorage.call(this, value.toLowerCase())) return true;
            return typeof value === 'number' && !isNaN(value) && (!this.noZero || value >= 0);
        }
    }
    get DefaultValue() {
        return function () {
            throw 0;
        }
    }
    get IsDefaultValue() {
        return function (value) {
            throw value === 0;
        }
    }
    get StringRegx() {
        if (this.dot > 0) return [/^-?[1-9]\d*$/, /^-?([1-9]\d*\.\d*|0\.\d*[1-9]\d*|0?\.0+|0)$/];
        return /^-?[1-9]\d*$/;
    }
    get Storage() {
        return function (key) {
            return Storage[key]();
        }
    }

    get IsStorage() {
        return function (key) {
            return Storage[key] !== undefined;
        }
    }

    get FormatHandler() {
        return (value, NullToEmpty) => {
            if (typeof value === 'number') {
                if (!isNaN(value)) {
                    return this.BaseInt > 1 ? Math.round(value * this.BaseInt) / this.BaseInt : Math.round(value);
                }
            }
            else if (typeof value === 'string') {
                return this.ConvertFromString(value, NullToEmpty);
            }
            return NullToEmpty ? this.Empty.call(this) : undefined;
        }
    }
    get CountHandler() {
        const c = (l, args) => {
            if (args.length === 0) return;
            if (args.length === 1) return args[0];
            let v = args[0];
            for (let i = 1; i < args.length; i++) {
                if (v !== undefined && !isNaN(v)) { v = Count(v, args[i], l); }
            }
            if (v && !isNaN(v)) {
                return this.BaseInt > 1 ? Math.round(v * this.BaseInt) / this.BaseInt : Math.round(v);
            }
            return v;
        }
        return {
            "+": function () { return c('+', arguments); },
            "-": function () { return c('-', arguments); },
            "*": function () { return c('*', arguments); },
            "/": function () { return c('/', arguments); }
        }
    }
    get CompareHanlder() {
        return {
            ">": function (a, b) { return a > b; },
            ">=": function (a, b) { return a >= b; },
            "=": function (a, b) { return a === b; },
            "<": function (a, b) { return a < b; },
            "<=": function (a, b) { return a <= b; }
        }
    }
}
module.exports = number;